// Build: g++ lhe_reader_TP.cpp -o lhe_reader_TP -I`root-config --incdir` `root-config --libs`

#include <cmath>
#include <string>
#include <cstring>
#include <iostream>
#include <sstream>
#include <fstream>
#include <TH1F.h>
#include <TFile.h>
#include <TLorentzVector.h>
using namespace std;

// Pour une description du format leshouches
// hep-ph/0609017
//
// pour chaque evenement
// une ligne générale : NbPart idprocess poids scale alpha_em alpha_s
// pour chaque particule : id status mere1 mere2 couleur1 couleur2 px py pz E m lifetime spin  


int main(int argc, char **argv) {

  if (argc != 2) {
    cout << "Missing argument. Expected LHE filename without '.lhe'"<< endl;
    exit(-5);
  }

  string basename=argv[1];

  string lhefname = basename+".lhe";
  string rootfname = basename+".root";

  string tt;
  int event=0;
  int npart,idprocess;
  double weight,scale,alpha_em,alpha_s;

  TH1::SetDefaultSumw2(true);


// --- TP : insert here histogram definition



 
  ifstream ff(lhefname.c_str(),ios::in); //ouverture du fichier .lhe
  long line = 0;
  while(!ff.eof()) {
    std::stringstream buffer;
    ff>>tt;
    buffer << tt << std::endl;
    line++;
    if(tt=="<event>") {
      ff>>npart>>idprocess>>weight>>scale>>alpha_em>>alpha_s; //event definition
      buffer << npart << " " << idprocess << " " << weight << " " << scale << " " << alpha_em << " " << alpha_s << std::endl;
      line++;
      event++;
      
      if (event%1000==0) cout << "reading event "<< event << endl;
      int top=-1,topbar=-1;
      int *Id      = new int[npart+1]; 
      int *Status  = new int[npart+1];
      int *Mother1 = new int[npart+1]; 
      int *Mother2 = new int[npart+1]; 
      int *Color1  = new int[npart+1];
      int *Color2  = new int[npart+1];
      double *px = new double[npart+1];
      double *py = new double[npart+1];
      double *pz = new double[npart+1];
      double *E = new double[npart+1];
      double *m = new double[npart+1];
      double *lifetime = new double[npart+1];
      double *spin = new double[npart+1];
      TLorentzVector **v = new TLorentzVector*[npart+1];
      // in lhe files, the first line is number 1, so fill unused array [0] with a crazy value;
      Id[0]= -99999; 
      Status[0]= -99999;
      Mother1[0]= -99999; 
      Mother2[0]= -99999; 
      Color1[0]= -99999;
      Color2[0]= -99999;
      px[0]= -99999;
      py[0]= -99999;
      pz[0]= -99999;
      E[0]= -99999;
      m[0]= -99999;
      lifetime[0]= -99999;
      spin[0]= -99999;

      // TP : ici déclaration des variables globales
      int nneutrinos=0;
      int nchargedlept=0;
      int nlightquarks=0;
      int nextrajet=0;

      for (int i=1 ; i<npart+1 ; i++) { //start at one
        ff >> Id[i] >> Status[i] >> Mother1[i] >> Mother2[i] >> Color1[i] >> Color2[i]  
           >> px[i] >> py[i] >> pz[i] >> E[i] >> m[i] >> lifetime[i] >> spin[i] ;
        buffer << Id[i] << " " << Status[i] << " " << std::endl;
        // cout << Id[i] << " " << Status[i] << " " << Mother1[i] << " "<< Mother2[i] << " "<< Color1[i] << " "<< Color2[i]  << " "<< px[i] << " "<< py[i] << " "<< pz[i] << " "<< E[i] << " "<< m[i] << " "<< lifetime[i] << " "<< spin[i] << endl;

        line++;
        v[i] = new TLorentzVector(px[i], py[i], pz[i], E[i]);
// --- TP : ici opération sur les particules


        if (Status[i]==-1) continue; // status -1 = initial quark ==> skip
        if (Id[i]==6)  top=i;
        if (Id[i]==-6) topbar=i;

        if (abs(Id[Mother1[i]])==24 || abs(Id[Mother1[i]])==6) { // mother = W or top
          int id = abs(Id[i]);
          if ( id==11 || id==13 || id==15 ) nchargedlept++;
          if ( id==12 || id==14 || id==16 ) nneutrinos++; 
          if (id<5) nlightquarks++;
          if ( Status[i]==1 && Mother1[i]==1 && Mother2[i]==2 && (abs(Id[i])<6 || Id[i]==21) ) nextrajet++;
        }
      }
      if (top == -1 || topbar == -1) {
        std::cout << "Warning: no tt~ in this event (line " << line << ")" << std::endl;
        std::cout << buffer.str() << std::endl;
        continue;
      }
      
// -- TP insérer ici le code pour remplir les histogrammes


      ff>>tt;
      line++;
      delete[] Id;
      delete[] Status;
      delete[] Mother1;
      delete[] Mother2;
      delete[] Color1;
      delete[] Color2;
      delete[] px;
      delete[] py;
      delete[] pz;
      delete[] E;
      delete[] m;
      delete[] lifetime;
      delete[] spin;
       for (int k=1 ; k<npart+1 ; delete v[k++]);
    }
  } // end while !eof

  cout << " Total number of events --> " << event << endl;
// --- TP ici  écrire les histogrammes dans le fichier root
  // TFile *rootfile = new TFile(rootfname.c_str(),"recreate");
  // rootfile->Close();

  exit(0);
}
